package miolnet.guconnectivity;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.schmizz.sshj.SSHClient;
import net.schmizz.sshj.common.IOUtils;
import net.schmizz.sshj.connection.ConnectionException;
import net.schmizz.sshj.connection.channel.direct.Session;
import net.schmizz.sshj.connection.channel.direct.Session.Command;
import net.schmizz.sshj.transport.TransportException;
import net.schmizz.sshj.transport.verification.HostKeyVerifier;
import net.schmizz.sshj.userauth.UserAuthException;

/**
 * This is implementation of GUConnectivity which uses the SSH protocol for connection. This depends
 * on external SSHJ library. 
 * 
 * @author Olmr Miloš
 *
 */
public class GUConnectivitySSH implements GUConnectivity {
	
	private static final Logger LOGGER = Logger.getLogger(GUConnectivitySSH.class.getName());
	
	private static final int TIMEOUT = 5000;
	private static final int READRETRYCOUNT = 3;
	
	private static final String W1BUSPATH = "/sys/bus/w1/devices";
	private static final String W1READINTERFACE = "/w1_slave";
	private static final String PREFIX = "28-";
	private static final int USCLENGTH = 15;
	
	final SSHClient ssh = new SSHClient();
	
	public GUConnectivitySSH() {
		ssh.addHostKeyVerifier(new HostKeyVerifier() {
			@Override
			public boolean verify(String arg0, int arg1, java.security.PublicKey arg2) {
				return true;
			}
		});
		ssh.setTimeout(TIMEOUT);
	}

	@Override
	public boolean connect(final String ip, final String username, final String password) throws IOException, AuthenticationFailedException {
		if (ip == null || username == null || password == null) {
			return false;
		}
		ssh.connect(ip);
		try {
			ssh.authPassword(username, password.toCharArray());
		} catch(UserAuthException e) {
			throw new AuthenticationFailedException(e);
		}
		return true;
	}
	
	private Set<String> getSensorsSet() throws IOException, ConnectionException, TransportException {
		final Set<String> sensors = new HashSet<>();
		final Session sess = ssh.startSession();
		final Command cmd = sess.exec("ls " + W1BUSPATH);
		final String w1 = IOUtils.readFully(cmd.getInputStream()).toString();
		sess.close();
		String[] devices = w1.split("\n");
		for (int i = 0; i < devices.length; i++) {
			if (devices[i].length() == USCLENGTH && devices[i].startsWith(PREFIX)) {
				sensors.add(devices[i]);
			}
		}
		return sensors;
	}
	
	private String getSensorValue(final String usc) throws IOException, ConnectionException, TransportException, CRCCheckFailedException {
		String fullOutput = "";
		for (int i = 0; i < READRETRYCOUNT && fullOutput.indexOf("YES") == -1; i++) {
			final Session sess = ssh.startSession();
			final Command cmd = sess.exec("cat " + W1BUSPATH + "/" + usc + W1READINTERFACE);
			fullOutput = IOUtils.readFully(cmd.getInputStream()).toString();
			sess.close();
		}
		if (fullOutput.indexOf("YES") == -1) {
			throw new CRCCheckFailedException(usc, fullOutput);
		}
		return fullOutput.split("t=")[1].trim();
	}

	@Override
	public Map<String, Float> gatherTemperatures() throws IOException {
		final Map<String, Float> sensorsAndValues = new HashMap<>();
		final Set<String> sensors = getSensorsSet();
		for (final String usc : sensors) {
			try {
				String value = getSensorValue(usc);
				if (value != null) {
					float temp = Float.parseFloat(value);
					//temp output is in milidegrees
					temp = temp / 1000;
					sensorsAndValues.put(usc, temp);
				}
			} catch (CRCCheckFailedException | NumberFormatException e) {
				LOGGER.log(Level.WARNING, e.getLocalizedMessage(), e);
			}
		}
		return sensorsAndValues;
	}
	
	@Override
	protected void finalize() throws Throwable {
		close();
		super.finalize();
	}

	@Override
	public void close() throws IOException {
		ssh.disconnect();
	}

}
