package miolnet.services;

import java.io.IOException;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.ejb.LocalBean;
import javax.ejb.Schedule;
import javax.ejb.Stateless;
import javax.inject.Inject;

import miolnet.guconnectivity.AuthenticationFailedException;
import miolnet.guconnectivity.GUConnectivity;
import miolnet.guconnectivity.GUConnectivitySSH;
import miolnet.model.Gather;
import miolnet.model.GatheringUnit;
import miolnet.model.Sensor;

/**
 * Session Bean implementation class GatheringService
 */
@Stateless
@LocalBean
public class GatheringService {
	
	private static final Logger LOGGER = Logger.getLogger(GatheringService.class.getName());
	
	@Inject
	private GatheringUnitServiceLocal gUnitService;
	
	@Inject
	private SensorServiceLocal sensorService;
	
	@Inject
	private TemperatureServiceLocal tempService;

    /**
     * Default constructor. 
     */
    public GatheringService() {
    }
    
    private GUConnectivity getProperConnectivity(final GatheringUnit gu) {
		switch (gu.getLoginDetails().getProtocol()) {
			case SSH:
				return new GUConnectivitySSH();
			default:
				LOGGER.log(Level.WARNING, "GatheringUnit ID: " + gu.getId() + " has invalid protocol.");
		}
		return null;
    }
    
    private void processData(final Map<String, Float> data, final Gather gather, final GatheringUnit gu) {
    	final Iterator<Entry<String, Float>> it = data.entrySet().iterator();
    	while (it.hasNext()) {
    		final Entry<String, Float> entry = it.next();
    		Sensor s = sensorService.getByUsc(entry.getKey());
    		if (s == null) {
    			s = sensorService.createSensor(entry.getKey(), gu);
    		}
    		else {
    			if (!gu.equals(s.getGatheringUnit())) {
    				final GatheringUnit prevGU = s.getGatheringUnit();
    				prevGU.getSensors().remove(s);
    				s.setGatheringUnit(gu);
    				gu.getSensors().add(s);
    				gUnitService.updateGatheringUnit(prevGU);
    				gUnitService.updateGatheringUnit(gu);
    			}
    		}
    		tempService.addTemperature(entry.getValue(), s, gather);
			s.setAvailable(true);
			s.setLastGatherTime(gather.getGatherTime());
			sensorService.updateSensor(s);
    	}
    }
    
    private Map<String, Float> getDataFromGatheringUnit(final GatheringUnit gu) {
		final GUConnectivity con = getProperConnectivity(gu);
		if (con != null) {
			try {
				if (con.connect(gu.getIp(), gu.getLoginDetails().getUsername(), gu.getLoginDetails().getPassword())) {
					final Map<String, Float> data =  con.gatherTemperatures();
					try {
						con.close();
					} catch (IOException e) {
						LOGGER.log(Level.WARNING, e.getMessage(), e);
					}
					return data;
				}
			} catch (IOException | AuthenticationFailedException e) {
				LOGGER.log(Level.WARNING, e.getMessage(), e);
			} finally {
				try {
					con.close();
				} catch (IOException e) {
					LOGGER.log(Level.WARNING, e.getMessage(), e);
				}
			}
		}
		return null;
    }
    
    @Schedule(minute = "*/5", hour = "*")
    public void gatherDataFromGatheringUnits() {
    	final Gather gather = new Gather(new Date());
    	final List<GatheringUnit> gatheringUnits = gUnitService.getAll();
    	for (final GatheringUnit gu : gatheringUnits) {
    		final Map<String, Float> data = getDataFromGatheringUnit(gu);
    		if (data != null) {
    			LOGGER.log(Level.INFO, "Sensors GatheringUnit ID "+gu.getId()+" count: "+data.size());
    			processData(data, gather, gu);
    			List<Sensor> sensors = sensorService.getAllWithoutThisLastGatherTime(gather.getGatherTime());
    			for (Sensor s : sensors) {
    				s.setAvailable(false);
    				sensorService.updateSensor(s);
    			}
    			gu.setLastGatherTime(gather.getGatherTime());
    			gu.setAvailable(true);
    		}
    		else {
    			gu.setAvailable(false);
    		}
    		gUnitService.updateGatheringUnit(gu);
    	}
    }

}
