package miolnet.services;

import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.TypedQuery;
import javax.validation.ConstraintViolation;
import javax.validation.Validator;

import miolnet.model.Gather;
import miolnet.model.Sensor;
import miolnet.model.Temperature;

/**
 * Session Bean implementation class TemperatureService
 */
@Stateless
public class TemperatureService implements TemperatureServiceLocal {

	private static final Logger LOGGER = Logger.getLogger(TemperatureService.class.getName());
	
	@Inject
	private EntityManager em;
	
	@Inject
	private Validator validator;
	
    /**
     * Default constructor. 
     */
    public TemperatureService() {
    }

	@Override
	public Temperature addTemperature(final Temperature temp) {
		em.persist(temp);
		return temp;
	}

	@Override
	public Temperature addTemperature(float temp, final Sensor sensor, final Gather gather) {
		final Temperature t = new Temperature(temp, gather, sensor);
		final Set<ConstraintViolation<Temperature>> violations = validator.validate(t);
		if (violations.isEmpty()) {
			em.persist(t);
		} 
		else {
			LOGGER.log(Level.WARNING, "Constraint violation during addTemperature. " + violations.toString());
		}
		return t;
	}

	@Override
	public Temperature getCurrentTemperature(final Sensor sensor) {
		final TypedQuery<Temperature> query = em.createNamedQuery("findSensorLastTemps", Temperature.class);
		query.setParameter("sensorId", sensor.getId());
		query.setMaxResults(1);
		Temperature t = null;
		try {
			t = query.getSingleResult();
		} catch(NoResultException e) {
		}
		return t;
	}
	
	@Override
	public Temperature getPreviousTemperature(Sensor sensor) {
		final TypedQuery<Temperature> query = em.createNamedQuery("findSensorLastTemps", Temperature.class);
		query.setParameter("sensorId", sensor.getId());
		query.setMaxResults(2);
		Temperature t = null;
		List<Temperature> temps = query.getResultList();
		if (temps != null && temps.size() >= 2) {
			t = temps.get(1);
		}
		return t;
	}

	@Override
	public List<Temperature> getDayTemperatures(Sensor sensor) {
		final Date d = new Date();
		return getCustomIntervalTemperatures(sensor, new Date(d.getTime()-86400000), d);
	}

	@Override
	public List<Temperature> getWeekTemperatures(Sensor sensor) { //prumer za pulhodiny
		final Date d = new Date();
		return getCustomIntervalTemperatures(sensor, new Date(d.getTime()-604800000), d); 
	}

	@Override
	public List<Temperature> getMonthTemperatures(Sensor sensor) { //prumer za kazde dve hodiny
		final Date d = new Date();
		return getCustomIntervalTemperatures(sensor, new Date(d.getTime()-2419200000L), d);
	}

	@Override
	public List<Temperature> getYearTemperatures(Sensor sensor) { //prumer za den
		final Date d = new Date();
		return getCustomIntervalTemperatures(sensor, new Date(d.getTime()-31536000000L), d);
	}

	@Override
	public List<Temperature> getCustomIntervalTemperatures(Sensor sensor, Date from, Date to) { //optimalizace
		final TypedQuery<Temperature> query = em.createNamedQuery("findSensorIntervalTemps", Temperature.class);
		query.setParameter("sensorId", sensor.getId());
		query.setParameter("from", from);
		query.setParameter("to", to);
		return query.getResultList();
	}

	@Override
	public float getAverageTemperature(Sensor sensor, Date from, Date to) {
		final TypedQuery<Double> query = em.createNamedQuery("avgSensorIntervalTemp", Double.class);
		query.setParameter("sensorId", sensor.getId());
		query.setParameter("from", from);
		query.setParameter("to", to);
		final Double avg = query.getSingleResult();
		if(avg != null) {
			return avg.floatValue();
		}
		final Temperature t = getCurrentTemperature(sensor);
		if (t != null) {
			return t.getTemp();
		}
		return 0;
	}

	@Override
	public Temperature getMaxTemperature(Sensor sensor, Date from, Date to) {
		final TypedQuery<Float> query = em.createNamedQuery("maxSensorIntervalVal", Float.class)
				.setParameter("sensorId", sensor.getId())
				.setParameter("from", from)
				.setParameter("to", to);
		final Float max = query.getSingleResult();
		return getLastTempOfTemps(sensor, max, from, to);
	}

	@Override
	public Temperature getMinTemperature(Sensor sensor, Date from, Date to) {
		final TypedQuery<Float> query = em.createNamedQuery("minSensorIntervalVal", Float.class)
				.setParameter("sensorId", sensor.getId())
				.setParameter("from", from)
				.setParameter("to", to);
		final Float min = query.getSingleResult();
		return getLastTempOfTemps(sensor, min, from, to);
	}
	
	private Temperature getLastTempOfTemps(Sensor sensor, float temp, Date from, Date to) {
		final TypedQuery<Temperature> query = em.createNamedQuery("findSensorIntervalTemp", Temperature.class)
				.setParameter("sensorId", sensor.getId())
				.setParameter("temp", temp)
				.setParameter("from", from)
				.setParameter("to", to);
		final List<Temperature> list = query.getResultList();
		if (!list.isEmpty()) {
			return list.get(0);
		}
		return null;
	}
	
}
