package miolnet.modules.heating.services;

import java.text.ParseException;
import java.time.LocalDate;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import miolnet.model.Module;
import miolnet.model.ModuleSensor;
import miolnet.model.Record;
import miolnet.model.Setting;
import miolnet.modules.heating.HeatingModule;
import miolnet.modules.heating.model.SensorData.Type;
import miolnet.services.ModuleServiceLocal;
import miolnet.services.RecordServiceLocal;

/**
 * Session Bean implementation class HeatingControlService
 */
@Stateless
public class HeatingControlService implements HeatingControlServiceLocal {

	@EJB
	private RecordServiceLocal recordService;
	
	@EJB
	private ModuleServiceLocal moduleService;
	
    /**
     * Default constructor. 
     */
    public HeatingControlService() {
    }
    
    @PostConstruct
    private void init() {
    }

	@Override
	public String getMTag() {
		return HeatingModule.TAG;
	}

	@Override
	public boolean isCurrentlyHeatingOn() {
		if (!getSettings().get("heatingEnabled").equals("true")) {
			return false;
		}
		final List<Record> records = recordService.getLastNRecordsWithValue(getHeatingModule(), "Heating state changed", 1);
		if (!records.isEmpty()) {
			return records.get(0).isStatus();
		}
		return false; //probably no records, because heating has not been on yet
	}

	@Override
	public int calculateHeatingDuration() {
		if (!getSettings().get("heatingEnabled").equals("true")) {
			return 0;
		}
		final List<Record> records = recordService.getLastNRecordsWithValue(getHeatingModule(), "Heating state changed", 1);
		if (!records.isEmpty()) {
			long time = new Date().getTime()-records.get(0).getRecordtime().getTime();
			//time is more than a week - return 0
			return time > 604800000 ? 0 : (int)(time);
		}
		return 0; //probably no records, because heating has not been on yet
	}

	@Override
	public float getCurrentTemp() {
		return HeatingModule.DISPLAY_TEMP;
	}

	@Override
	public Map<String, String> getSettings() {
		final Module heatingModule = getHeatingModule();
		Map<String, String> settings = new HashMap<>(heatingModule.getSettings().size());
		for(Setting s : heatingModule.getSettings()) {
			if (!s.getName().equals("heatingOutdoorSensorUSC") &&
				!s.getName().equals("heatingHeatNow") &&
				!s.getName().equals("heatingHeatNowDuration")) {
				settings.put(s.getName(), s.getValue());
			}
		}
		return settings;
	}

	@Override
	public float getTempFromRecordValue(final String value) throws ParseException {
		String[] values = value.split(":");
		if (value.contains("temp") && values.length==2) {
			String strTemp = values[1].trim();
			try {
				return Float.parseFloat(strTemp);
			} catch(NumberFormatException e) {
				throw new ParseException("Bad record! NumberFormatException happened!", value.length()-1);
			}
		}
		throw new ParseException("Bad record! No colon found to split the record and extract temperature!", value.length());
	}

	@Override
	public Type getSensorType(final String usc) {
		final Module heatingModule = getHeatingModule();
		for (Setting s : heatingModule.getSettings()) {
			if (s.getName().equals("heatingOutdoorSensorUSC") && s.getValue().equals(usc)) {
				return Type.OUTDOOR;
			}
		}
		return Type.INDOOR;
	}

	@Override
	public boolean heatNow(int duration) {
		final Module heatingModule = getHeatingModule();
		for (Setting s : heatingModule.getSettings()) {
			if (s.getName().equals("heatingEnabled")&&!s.getValue().equals("true")) {
				return false;
			}
			if (s.getName().equals("heatingHeatNow")) {
				s.setValue("true");
			}
			if (s.getName().equals("heatingHeatNowDuration")) {
				s.setValue(String.valueOf(duration));
			}
		}
		moduleService.updateModule(heatingModule);
		recordService.createRecord(new Date(), "Heating heat now", true, heatingModule);
		return true;
	}

	@Override
	public void saveSettings(boolean heatingEnabled, float difference, float outdoorTempLimit) {
		final Module heatingModule = getHeatingModule();
		for (Setting s : heatingModule.getSettings()) {
			if (s.getName().equals("heatingEnabled")) {
				s.setValue(String.valueOf(heatingEnabled));
			}
			if (s.getName().equals("heatingHysteresisDifference")) {
				s.setValue(String.valueOf(difference));
			}
			if (s.getName().equals("heatingOutdoorTempLimit")) {
				s.setValue(String.valueOf(outdoorTempLimit));
			}
		}
		moduleService.updateModule(heatingModule);
	}

	@Override
	public List<ModuleSensor> getSensors() {
		return getHeatingModule().getModuleSensors();
	}

	@Override
	public List<Record> getHistory(int numOfRecords) {
		return recordService.getLastNRecordsWithValue(getHeatingModule(), "Heating state changed", numOfRecords);
	}

	@Override
	public boolean isHeatNowOverlapping() {
		final Module heatingModule = getHeatingModule();
		List<Record> records = recordService.getLastNRecordsWithValue(heatingModule, "Heating heat now", 1);
		if (records.size()>0) {
			for (Setting s : heatingModule.getSettings()) {
				if (s.getName().equals("heatingHeatNowDuration")) {
					try {
						int duration = Integer.valueOf(s.getValue());
						return new Date().getTime()-records.get(0).getRecordtime().getTime()>=duration*60000;
					} catch(NumberFormatException e) {
					}
				}
			}
		}
		return false;
	}

	@Override
	public void expireHeatNow() {
		final Module heatingModule = getHeatingModule();
		for (Setting s : heatingModule.getSettings()) {
			if (s.getName().equals("heatingHeatNow")) {
				s.setValue("false");
			}
		}
		moduleService.updateModule(heatingModule);
		recordService.createRecord(new Date(), "Heating heat now", false, heatingModule);
	}
	
	private Module getHeatingModule() {
		return moduleService.getByTag(HeatingModule.TAG);
	}

	@Override
	public boolean isHeatNowOn() {
		final Module heatingModule = getHeatingModule();
		for (Setting s : heatingModule.getSettings()) {
			if (s.getName().equals("heatingHeatNow")) {
				return "true".equals(s.getValue());
			}
		}
		return false;
	}
	
	private int calculateDifferenceBetweenDatesInMinutes(final Date date1, final Date date2) {
		return Math.abs((int)(date1.getTime()-date2.getTime()));
	}

	@Override
	public int calculateOverallDayHeatingTime(final LocalDate day) {
		final Module heatingModule = getHeatingModule();
		final Date dateStart = Date.from(day.atTime(LocalTime.MIN).atZone(ZoneId.systemDefault()).toInstant());
		final Date dateEnd = Date.from(day.atTime(LocalTime.MAX).atZone(ZoneId.systemDefault()).toInstant());
		final List<Record> records = recordService.getRecordsWithValueInDateInterval(heatingModule, "Heating state changed", dateStart, dateEnd);
		if (records == null || records.isEmpty()) {
			return 0;
		}
		int minutes = 0;
		final Iterator<Record> it = records.iterator();
		Record recordA = it.next();
		if (!recordA.isStatus()) {
			//calculate time between midnight and this record
			minutes += calculateDifferenceBetweenDatesInMinutes(recordA.getRecordtime(), dateStart);
		}
		Record recordB = recordA;
		while (it.hasNext()) {
			 recordB = it.next();
			 if (recordA.isStatus()) {
				 //calculate heating time
				 minutes += calculateDifferenceBetweenDatesInMinutes(recordB.getRecordtime(), recordA.getRecordtime());
			 }
			 recordA = recordB;
		}
		if (recordA.isStatus()) {
			//calculate time from heating start record (if it is last) till end of the day or till now
			if (LocalDate.now().equals(day)) {
				//the day is today - till now
				minutes += calculateDifferenceBetweenDatesInMinutes(new Date(), recordA.getRecordtime());
			}
			else {
				//till end of the day
				minutes += calculateDifferenceBetweenDatesInMinutes(dateEnd, recordA.getRecordtime());
			}
		}
		return minutes;
	}

}
