package miolnet.modules.heating.services;

import java.time.LocalTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.inject.Inject;

import miolnet.model.Module;
import miolnet.model.ModuleSensor;
import miolnet.model.Setting;
import miolnet.modules.heating.HeatingModule;
import miolnet.modules.heating.model.DayPeriod;
import miolnet.modules.heating.model.SensorData;
import miolnet.services.RecordServiceLocal;
import miolnet.services.TemperatureServiceLocal;

/**
 * Session Bean implementation class HeatingDataService
 */
@Stateless(mappedName="HeatingDataService")
public class HeatingDataService implements HeatingDataServiceRemote {
	
	@Inject
	private DayPeriodServiceLocal dayPeriodService;
	
	@EJB
	private TemperatureServiceLocal temperatureService;
	
	@EJB
	private RecordServiceLocal recordService;
	
	@Inject
	private HeatingModule heatingModule;
	
	@Inject
	private HeatingControlServiceLocal controlService;

    /**
     * Default constructor. 
     */
    public HeatingDataService() {
    }

	@Override
	public Map<String, String> getSettings() {
		if (controlService.isHeatNowOverlapping()) {
			controlService.expireHeatNow();
		}
		final Map<String, String> settings = new HashMap<>();
		final Module module = heatingModule.getThisModule();
		if (module != null) {
			for (Setting s : module.getSettings()) {
				if ("heatingOutdoorSensorUSC".equals(s.getName())) {
					continue;
				}
				settings.put(s.getName(), s.getValue());
			}
		}
		return settings;
	}
	
	private Set<String> getOutdoorSensorsUSCs(final Module module) {
		final Set<String> outdoorSensorsUSCs = new HashSet<>();
		for (Setting s : module.getSettings()) {
			if ("heatingOutdoorSensorUSC".equals(s.getName())) {
				outdoorSensorsUSCs.add(s.getValue());
			}
		}
		return outdoorSensorsUSCs;
	}

	@Override
	public List<SensorData> getData() {
		final List<SensorData> data = new ArrayList<>();
		final Module module = heatingModule.getThisModule();
		if (module != null) {
			final Set<String> outdoorSensors = getOutdoorSensorsUSCs(module);
			for (ModuleSensor ms : module.getModuleSensors()) {
				data.add(new SensorData(
						outdoorSensors.contains(ms.getSensor().getUsc())?
								SensorData.Type.OUTDOOR:
								SensorData.Type.INDOOR, 
						ms.getPriority(), 
						temperatureService.getCurrentTemperature(ms.getSensor()).getTemp()));
			}
		}
		return data;
	}

	@Override
	public LocalTime getServerTime() {
		return LocalTime.now();
	}

	@Override
	public List<DayPeriod> getDayPeriods() {
		return dayPeriodService.getAll();
	}
	
	@Override
	public void sendHeatingState(boolean isHeatingOn) {
		final Module module = heatingModule.getThisModule();
		if (module != null) {
			recordService.createRecord(new Date(), "Heating state changed", isHeatingOn, module);
		}
	}

	@Override
	public void sendHeatingState(boolean isHeatingOn, float temp) {
		final Module module = heatingModule.getThisModule();
		if (module != null) {
			recordService.createRecord(new Date(), "Heating state changed at temp: " + temp, isHeatingOn, module);
		}
	}

	@Override
	public void sendCalculatedAvgTemp(float temp) {
		HeatingModule.DISPLAY_TEMP = temp;
	}

}
