package miolnet.client;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Timer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import miolnet.client.thermoregulator.Configuration;
import miolnet.client.thermoregulator.Configuration.ThermoRegulatorMode;
import miolnet.client.thermoregulator.GpioHeatingSwitch;
import miolnet.client.thermoregulator.HeatingSwitch;
import miolnet.client.thermoregulator.LocalLinux1wSensor;
import miolnet.client.thermoregulator.Sensor;
import miolnet.client.thermoregulator.ThermoRegulatorAlg;
import miolnet.modules.heating.services.HeatingDataServiceRemote;

public class ThermoRegulator {
	
	public static final Logger LOGGER = Logger.getLogger(ThermoRegulator.class.getName());
	
	public static final String CONFIG = "config.properties";
	
	public static Configuration loadConfiguration(final String filename) {
		try {
			Configuration conf = Configuration.load(filename);
			return conf;
		} catch (FileNotFoundException e) {
			LOGGER.log(Level.WARNING, e.toString(), e);
			Configuration config = new Configuration();
			try {
				Configuration.save(config, filename);
			} catch (IOException e1) {
				LOGGER.log(Level.WARNING, e1.toString(), e1);
			}
			return config;
		} catch (IOException e) {
			LOGGER.log(Level.WARNING, e.toString(), e);
			return new Configuration();
		}
	}

	public static void main(String[] args) {
		LOGGER.log(Level.INFO, "Starting thermoregulator...");
		
		final String configSource = args.length==0?CONFIG:args[0];
		
		LOGGER.log(Level.INFO, "Loading configuration from: "+configSource);
		
		final Configuration config = loadConfiguration(configSource);
		System.setProperty("org.omg.CORBA.ORBInitialHost", config.getServerIp());
	    System.setProperty("org.omg.CORBA.ORBInitialPort", config.getServerPort());
		
		final Sensor sensor = new LocalLinux1wSensor(config.getSensorUSC());
		
		boolean clientFailCondition = config.getServerIp().isEmpty() || config.getServerPort().isEmpty();
		boolean standAloneFailCondition = config.getSensorUSC().isEmpty() || !sensor.isPresent();
		if (config.getMode() == ThermoRegulatorMode.CLIENT_ONLY && clientFailCondition ||
			config.getMode() == ThermoRegulatorMode.STAND_ALONE && standAloneFailCondition ||
			config.getMode() == ThermoRegulatorMode.MIXED && clientFailCondition && standAloneFailCondition) {
			LOGGER.log(Level.SEVERE, "Wrong configuration! Thermoregulator can't work without some source of temp data.");
			return;
		}
		
		if (config.getMode() == ThermoRegulatorMode.MIXED && clientFailCondition) {
			LOGGER.log(Level.WARNING, "Wrong configuration for MIXED mode! You must define server IP and port. Changing mode to STAND_ALONE.");
			config.setMode(ThermoRegulatorMode.STAND_ALONE);
		}
		
		if (config.getMode() == ThermoRegulatorMode.MIXED && standAloneFailCondition) {
			LOGGER.log(Level.WARNING, "Wrong configuration for MIXED mode! You must define USC (unique serial code) of connected sensor. Changing mode to CLIENT_ONLY.");
			config.setMode(ThermoRegulatorMode.CLIENT_ONLY);
		}
		
		LOGGER.log(Level.INFO, "Entering HeatingSwitch initialization phase");
		
		//final HeatingSwitch heatingSwitch = new DebugHeatingSwitch();
		final HeatingSwitch heatingSwitch = new GpioHeatingSwitch(config.getGpioPin(), config.isGpioLowPinStateEqHeatingOn());
		
		LOGGER.log(Level.INFO, "Deploying Thermoregulator Algorithm");
		
		final ThermoRegulatorAlg alg = new ThermoRegulatorAlg(new HeatingDataServiceProvider() {
			
			@Override
			public HeatingDataServiceRemote getHeatingDataService() {
				InitialContext ctx;
				try {
					ctx = new InitialContext();
					Object obj = ctx.lookup("HeatingDataService");
					if (obj != null) {
						return (HeatingDataServiceRemote) PortableRemoteObject.narrow(obj, HeatingDataServiceRemote.class);
					}
				} catch (NamingException e) {
					LOGGER.log(Level.WARNING, e.getLocalizedMessage(), e);
				}
				return null;
			}
			
		}, sensor, heatingSwitch, config);
		Timer t = new Timer(alg.getClass().getName());
		t.schedule(alg, 0, config.getInterval());
		
		LOGGER.log(Level.INFO, "Thermoregulator is running.");
	}

}
