package miolnet.client.thermoregulator;

import java.time.LocalTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import miolnet.client.HeatingDataServiceProvider;
import miolnet.client.thermoregulator.Configuration.ThermoRegulatorMode;
import miolnet.client.thermoregulator.Sensor.ValueReadingException;
import miolnet.modules.heating.model.DayPeriod;
import miolnet.modules.heating.model.SensorData;
import miolnet.modules.heating.model.SensorData.Type;
import miolnet.modules.heating.services.HeatingDataServiceRemote;

/**
 * Algorithm (simple for STAND_ALONE and advanced for CLIENT mode) for ThermoRegulator
 * 
 * @author Olmr Miloš
 *
 */
public class ThermoRegulatorAlg extends TimerTask {
	
	public static final Logger LOGGER = Logger.getLogger(ThermoRegulatorAlg.class.getName());
	
	private final HeatingDataServiceProvider hdsProvider;
	private final Sensor sensor;
	private final HeatingSwitch heating;
	private final Configuration conf;
	
	private boolean connectionFailed;
	
	public ThermoRegulatorAlg(final HeatingDataServiceProvider hdsProvider, final Sensor sensor, final HeatingSwitch heating, final Configuration conf) {
		this.hdsProvider = hdsProvider;
		this.sensor = sensor;
		this.heating = heating;
		this.conf = conf;
		connectionFailed = true;
	}
	
	private float getCurrentRequestedTemp(final List<DayPeriod> dayPeriods, final LocalTime currentTime) {
		double avg = 0;
		//return appropriate requested temperature
		for (final DayPeriod dp : dayPeriods) {
			if (currentTime.isAfter(dp.getStartTime()) && currentTime.isBefore(dp.getEndTime())) {
				return dp.getTemperature();
			}
			avg += dp.getTemperature();
		}
		//PROBLEM happened
		LOGGER.log(Level.WARNING, "Current time doesn't fit into any DayPeriod! Returning average temperature of all DayPeriods.");
		if (dayPeriods.size() > 0) {
			return (float)(avg / dayPeriods.size());
		}
		LOGGER.log(Level.WARNING, "No DayPeriod defined! Returning temperature from config.");
		return conf.getTemp();
	}
	
	private void runAdvancedAlg(final HeatingDataServiceRemote dataService, float outdoorTempLimit) {
		//get necessary data from data service
		final List<DayPeriod> dayPeriods = dataService.getDayPeriods();
		if (dayPeriods == null) {
			LOGGER.log(Level.SEVERE, "Day periods from data service are NULL!");
			return;
		}
		
		final List<SensorData> data = dataService.getData();
		if (data == null) {
			LOGGER.log(Level.SEVERE, "Sensor data from data service are NULL!");
			return;
		}
		if (data.size() == 0) {
			LOGGER.log(Level.WARNING, "No sensor data!");
			return;
		}
		
		final LocalTime currentTime = dataService.getServerTime();
		if (currentTime == null) {
			LOGGER.log(Level.SEVERE, "Current time from data service is NULL!");
			return;
		}
		
		//get requested temp for current time
		float requestedTemp = getCurrentRequestedTemp(dayPeriods, currentTime);
		
		final List<SensorData> outdoorSensors = new ArrayList<>();
		double weightedAvgTemp = 0;
		int sum = 0;
		
		//calculate the weighted arithmetic mean
		for (final SensorData sensor : data) {
			//use only indoor temp sensors in this calculation
			if (sensor.getSensorType() == Type.OUTDOOR) {
				outdoorSensors.add(sensor);
				continue;
			}
			
			weightedAvgTemp += sensor.getPriority() * sensor.getValue();
			sum += sensor.getPriority();
		}
		
		if (sum == 0) {
			LOGGER.log(Level.WARNING, "Sum of sensor priorities equals 0!");
			//priority of all sensors is 0 (not use)
			weightedAvgTemp = 0;
			for (final SensorData sensor : data) {
				if (sensor.getSensorType() == Type.INDOOR) {
					weightedAvgTemp += sensor.getValue();
					sum++;
				}
			}
			//now sum cannot be 0, because it would mean that there is 0 sensors, but for
			//this situtation there is if at start of this method
		}

		//complete the calculation and send the value
		weightedAvgTemp /= sum;
		dataService.sendCalculatedAvgTemp((float)weightedAvgTemp);
		
		//check outdoor temp
		if (!outdoorSensors.isEmpty() && checkOutdoorTempExceeding(outdoorSensors, outdoorTempLimit)) {
			// stop heating
			if (heating.isOn()) {
				heating.turnOff();
				dataService.sendHeatingState(false, (float)weightedAvgTemp);
				LOGGER.log(Level.INFO, "Heating is now off! Temp: "+weightedAvgTemp);
			}
		}
			
		// calculate distance of average from requested temp
		float distance = Math.abs((float)(requestedTemp - weightedAvgTemp));

		// average is out of range that means to start or stop the heating
		if (distance >= conf.getDifference() / 2) {
			if (weightedAvgTemp < requestedTemp) {
				// start heating
				if (heating.isOff()) {
					heating.turnOn();
					dataService.sendHeatingState(true, (float)weightedAvgTemp);
					LOGGER.log(Level.INFO, "Heating is now on! Temp: "+weightedAvgTemp);
				}
			} else {
				// stop heating
				if (heating.isOn()) {
					heating.turnOff();
					dataService.sendHeatingState(false, (float)weightedAvgTemp);
					LOGGER.log(Level.INFO, "Heating is now off! Temp: "+weightedAvgTemp);
				}
			}
		}
	}
	
	private boolean checkOutdoorTempExceeding(final List<SensorData> outdoorSensors, float outdoorTempLimit) {
		if (outdoorSensors.isEmpty()) {
			return false;
		}
		float outdoorTemp = 0;
		for (final SensorData sd : outdoorSensors) {
			outdoorTemp += sd.getValue();
		}
		outdoorTemp /= outdoorSensors.size();
		return outdoorTemp > outdoorTempLimit;
	}

	private void runSimpleAlg(float temp) {
		float distance = Math.abs(conf.getTemp() - temp);
		
		//temp is out of range that means to start or stop the heating
		if (distance >= conf.getDifference()/2) {
			if (temp < conf.getTemp()) {
				//start heating
				if (heating.isOff()) {
					heating.turnOn();
					LOGGER.log(Level.INFO, "Heating is now on! Temp: "+temp);
				}
			}
			else {
				//stop heating
				if (heating.isOn()) {
					heating.turnOff();
					LOGGER.log(Level.INFO, "Heating is now off! Temp: "+temp);
				}
			}
		}
	}
	
	/**
	 * Advanced mode
	 * @throws ConnectionFailedException
	 */
	private void runAsClient() throws ConnectionFailedException {
		//get remote HeatingDataService and verify if it is not null (null means some connection problem) 
		final HeatingDataServiceRemote dataService = hdsProvider.getHeatingDataService();
		if (dataService == null) {
			throw new ConnectionFailedException();
		}
		
		//using service, get current remote settings
		final Map<String, String> settings = dataService.getSettings();
		
		if (settings == null) {
			LOGGER.log(Level.SEVERE, "Settings from data service are NULL!");
			return;
		}
		
		//parse heatingEnabled and verify if heating is enabled at all
		final String strEnabled = settings.get("heatingEnabled");
		if (strEnabled != null) {
			conf.setEnabled(Boolean.valueOf(strEnabled));
		}
		if (!conf.isEnabled()) {
			//if heating is disabled by user and it is turn on, then turn it off and end
			if (heating.isOn()) {
				heating.turnOff();
				dataService.sendHeatingState(false);
			}
			return;
		}
		
		//parse another setting
		final String strDifference = settings.get("heatingHysteresisDifference");
		if (strDifference != null) {
			try {
				conf.setDifference(Float.valueOf(strDifference));
			} catch(NumberFormatException e) {
				LOGGER.log(Level.WARNING, e.getLocalizedMessage(), e);
			}
		}
		
		if (settings.get("heatingHeatNow") != null && Boolean.valueOf(settings.get("heatingHeatNow"))) {
			//situation when user wants to just heat now (without any algorithm)
			if (heating.isOff()) {
				heating.turnOn();
				dataService.sendHeatingState(true);
			}
		}
		else {
			final String strOutdoorTempLimit = settings.get("heatingOutdoorTempLimit");
			float outdoorTempLimit = Float.MAX_VALUE;
			if (strOutdoorTempLimit != null) {
				try {
					outdoorTempLimit = Float.valueOf(strOutdoorTempLimit);
				} catch(NumberFormatException e) {
					LOGGER.log(Level.WARNING, e.getLocalizedMessage(), e);
				}
			}
			//standard way
			runAdvancedAlg(dataService, outdoorTempLimit);
		}
	}
	
	
	/**
	 * Simple thermostat mode
	 * @throws ValueReadingException 
	 */
	private void runStandAlone() throws ValueReadingException {
		if (!conf.isEnabled()) {
			//if heating is disabled by user and it is turn on, then turn it off and end
			if (heating.isOn()) {
				heating.turnOff();
			}
			return;
		}
		if (sensor != null && sensor.isPresent()) {
			runSimpleAlg(sensor.getValue());
		}
	}

	@Override
	public void run() {
		if (conf.getMode()==ThermoRegulatorMode.MIXED||conf.getMode()==ThermoRegulatorMode.CLIENT_ONLY) {
			try {
				runAsClient();
				if (connectionFailed) {
					connectionFailed = false;
					LOGGER.log(Level.INFO, "Successfully connected to application server: "+conf.getServerIp());
				}
			} catch(ConnectionFailedException e) {
				LOGGER.log(Level.WARNING, e.getMessage(), e);
				if (!connectionFailed) {
					connectionFailed = true;
					return;
				}
			}
		}
		if (conf.getMode()==ThermoRegulatorMode.MIXED && connectionFailed||
			conf.getMode()==ThermoRegulatorMode.STAND_ALONE) {
			try {
				runStandAlone();
			} catch (ValueReadingException e) {
				LOGGER.log(Level.WARNING, e.getLocalizedMessage(), e);
			}
		}
	}
	
	public class ConnectionFailedException extends Exception {

		private static final long serialVersionUID = 1L;
		
		public ConnectionFailedException() {
			super("Connection to application server failed!");
		}
		
	}

}
