package miolnet.web.controllers;

import java.text.ParseException;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;

import miolnet.model.ModuleSensor;
import miolnet.model.Record;
import miolnet.modules.heating.model.DayPeriod;
import miolnet.modules.heating.model.SensorData;
import miolnet.modules.heating.services.DayPeriodServiceLocal;
import miolnet.modules.heating.services.HeatingControlServiceLocal;
import miolnet.services.ModuleServiceLocal;

@ManagedBean
@RequestScoped
public class HeatingController {
	
	public static final Logger LOGGER = Logger.getLogger(HeatingController.class.getName());

	@EJB
	private HeatingControlServiceLocal heatingControlService;
	
	@EJB
	private DayPeriodServiceLocal dayperiodService;
	
	@EJB
	private ModuleServiceLocal moduleService;
	
	private String mtag;
	private String infoMessage;
	
	private boolean heating;
	private String heatingDuration;
	private float temp;
	private int heatNowDuration;
	
	private DayPeriod currentDayPeriod;
	private boolean heatNowOn;
	private String overallHeatingDuration;
	private String yesterdayOverallHeatingDuration;
	
	private List<DayPeriod> dayPeriods;
	private List<ModuleSensor> sensors;
	private List<Record> records;
	
	private boolean heatingEnabled;
	private float difference;
	private float outdoorTempLimit;
	
	@PostConstruct
	public void init() {
		mtag = heatingControlService.getMTag();
		infoMessage = "";
		heating = heatingControlService.isCurrentlyHeatingOn();
		int duration = heatingControlService.calculateHeatingDuration();
		heatingDuration = duration == 0 ? "" : createDurationString(duration);
		temp = heatingControlService.getCurrentTemp();
		heatNowDuration = 30;
		dayPeriods = dayperiodService.getAll();
		sensors = heatingControlService.getSensors();
		records = heatingControlService.getHistory(10);
		final Map<String, String> settings = heatingControlService.getSettings();
		heatingEnabled = Boolean.valueOf(settings.get("heatingEnabled"));
		try {
			difference = Float.valueOf(settings.get("heatingHysteresisDifference"));
		} catch(NumberFormatException e) {
			LOGGER.log(Level.WARNING, "Problem converting difference to float.", e);
			difference = 1;
		}
		try {
			outdoorTempLimit = Float.valueOf(settings.get("heatingOutdoorTempLimit"));
		} catch(NumberFormatException e) {
			LOGGER.log(Level.WARNING, "Problem converting outdoorTempLimit to float.", e);
			outdoorTempLimit = 13;
		}
		currentDayPeriod = dayperiodService.getByTime(LocalTime.now());
		heatNowOn = heatingControlService.isHeatNowOn();
		overallHeatingDuration = createDurationString(
				heatingControlService.calculateOverallDayHeatingTime(LocalDate.now()));
		yesterdayOverallHeatingDuration = createDurationString(
				heatingControlService.calculateOverallDayHeatingTime(LocalDate.now().minusDays(1)));
	}
	
	private String createDurationString(int duration) {
		int minutes = duration/60000;
		int hours = minutes/60;
		StringBuilder s = new StringBuilder();
		if (hours>0) {
			s = s.append(hours+"h ");
			minutes = minutes - hours*60;
		}
		s = s.append(minutes+"min");
		return s.toString();
	}
	
	public void saveSettings() {
		heatingControlService.saveSettings(heatingEnabled, difference, outdoorTempLimit);
	}
	
	public void heatNow() {
		if (heatingControlService.heatNow(heatNowDuration)) {
			infoMessage = String.format("Topení je nyní zapnuto uživatelem na %dmin! K sepnutí dojde, až regulátor "
					+ "tuto instrukci zaregistruje (na panelu se ukáže: „topí se: ANO“).", heatNowDuration);
		}
		else {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Vytápění není zapnuto!"));
		}
	}
	
	public Float getTempFromRecordValue(final String value) {
		try {
			return heatingControlService.getTempFromRecordValue(value);
		} catch (ParseException e) {
		}
		return null;
	}
	
	public String getSensorType(final String usc) {
		return heatingControlService.getSensorType(usc)==SensorData.Type.OUTDOOR?"Venkovní":"Vnitřní";
	}

	public String getInfoMessage() {
		return infoMessage;
	}

	public void setInfoMessage(String infoMessage) {
		this.infoMessage = infoMessage;
	}

	public String getMtag() {
		return mtag;
	}

	public void setMtag(String mtag) {
		this.mtag = mtag;
	}

	public List<DayPeriod> getDayPeriods() {
		return dayPeriods;
	}

	public void setDayPeriods(List<DayPeriod> dayPeriods) {
		this.dayPeriods = dayPeriods;
	}

	public List<ModuleSensor> getSensors() {
		return sensors;
	}

	public void setSensors(List<ModuleSensor> sensors) {
		this.sensors = sensors;
	}

	public List<Record> getRecords() {
		return records;
	}

	public void setRecords(List<Record> records) {
		this.records = records;
	}

	public boolean isHeating() {
		return heating;
	}

	public void setHeating(boolean heating) {
		this.heating = heating;
	}

	public String getHeatingDuration() {
		return heatingDuration;
	}

	public void setHeatingDuration(String heatingDuration) {
		this.heatingDuration = heatingDuration;
	}

	public float getTemp() {
		return temp;
	}

	public void setTemp(float temp) {
		this.temp = temp;
	}

	public int getHeatNowDuration() {
		return heatNowDuration;
	}

	public void setHeatNowDuration(int heatNowDuration) {
		this.heatNowDuration = heatNowDuration;
	}

	public boolean isHeatingEnabled() {
		return heatingEnabled;
	}

	public void setHeatingEnabled(boolean heatingEnabled) {
		this.heatingEnabled = heatingEnabled;
	}

	public float getDifference() {
		return difference;
	}

	public void setDifference(float difference) {
		this.difference = difference;
	}

	public float getOutdoorTempLimit() {
		return outdoorTempLimit;
	}

	public void setOutdoorTempLimit(float outdoorTempLimit) {
		this.outdoorTempLimit = outdoorTempLimit;
	}

	public DayPeriod getCurrentDayPeriod() {
		return currentDayPeriod;
	}

	public boolean isHeatNowOn() {
		return heatNowOn;
	}

	public String getOverallHeatingDuration() {
		return overallHeatingDuration;
	}

	public String getYesterdayOverallHeatingDuration() {
		return yesterdayOverallHeatingDuration;
	}

}
